/***************************************************************************
 *   Copyright (C) 2007 by Veli Mkinen                                    *
 *                                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
 ***************************************************************************/
 
#include "WaveletTree.h"

WaveletTree::WaveletTree(uchar *D, ulong n) {
    this->n = n;
    this->maxlevel = 8;
   // i-th entry = GetField(D,widthD,i);
    ulong i, k, value;
    ulong** bucket = new ulong*[maxlevel+1];

    ulong *wtree = new ulong[n*maxlevel/(8*sizeof(ulong))+1];
    for (i = 0; i < n*maxlevel/(8*sizeof(ulong))+1; i ++)
        wtree[i] = 0lu;

   // allocate memory for storing the locations of boudaries 
    for (k=0; k<=maxlevel; k++) {
        bucket[k] = (ulong *)new ulong[(ulong)1 << k];
        for (i=0; i<((ulong)1 << k); i++)
            bucket[k][i]=0;
    }

   // compute the sizes of buckets
    for (i=0; i<n; i++) {
        value = D[i];
        for (k=0; k<=maxlevel; k++)
            bucket[k][value >> (maxlevel-k)]++;
    }

   // cumulate the bucket sizes into boundary locations 
    for (k=0; k<=maxlevel; k++) {
        for (i=1; i<((ulong)1 << k); i++)
            bucket[k][i]+=bucket[k][i-1];
        for (i=((ulong)1 << k)-1; i>=1; i--)
            bucket[k][i]=bucket[k][i-1];
        bucket[k][0]=0;
    }

   //store the leaf information
    leaves = new ulong[((ulong)1 << maxlevel)+1];
    for (k=0; k<(ulong)1<<maxlevel; k++)
        leaves[k] = bucket[maxlevel][k];
    leaves[k]=n;

   // compute wtree using the information about boundaries 
    for (i=0; i<n; i++) {
        value = D[i];
        for (k=0; k<maxlevel; k++)
            Tools::SetField(wtree,1,k*n+(bucket[k][value >> (maxlevel-k)])++,(value>>(maxlevel-k-1)) & (ulong)1);
    }
//     for (k=0; k<maxlevel; k++) {
//         for (i=0; i<n; i++)   
//             std::cout << Tools::GetField(wtree,1,k*n+i);
//         std::cout << "\n";
//     }
   // Create rank-structure for wtree

    bitrank = new BitRank(wtree,n*maxlevel,true);
    /*ulong logn = ((Tools::FloorLog2(n*maxlevel)+1)/2)*2;
    bitrank = new CompressedBitVector(wtree,n*maxlevel,true,W*W,W/2,W/2);
    delete [] wtree;*/

   // deallocate boundaries
    for (k=0; k<=maxlevel; k++)
        delete[] bucket[k];
    delete[] bucket;
}

WaveletTree::WaveletTree(ulong *D, ulong n, unsigned widthD) {
   this->n = n;
   this->maxlevel = widthD;
   // i-th entry = GetField(D,widthD,i);
   ulong i, k, value;
   ulong** bucket = new ulong*[maxlevel+1];

   ulong *wtree = new ulong[n*maxlevel/(8*sizeof(ulong))+1];
   for (i = 0; i < n*maxlevel/(8*sizeof(ulong))+1; i ++)
       wtree[i] = 0lu;
   
   // allocate memory for storing the locations of boudaries 
   for (k=0; k<=maxlevel; k++) {
      bucket[k] = (ulong *)new ulong[(ulong)1 << k];
      for (i=0; i<((ulong)1 << k); i++)
         bucket[k][i]=0;
   }

   // compute the sizes of buckets
   for (i=0; i<n; i++) {
      value = Tools::GetField(D,maxlevel,i);
      for (k=0; k<=maxlevel; k++)
         bucket[k][value >> (maxlevel-k)]++;
   }

   // cumulate the bucket sizes into boundary locations 
   for (k=0; k<=maxlevel; k++) {
      for (i=1; i<((ulong)1 << k); i++)
         bucket[k][i]+=bucket[k][i-1];
      for (i=((ulong)1 << k)-1; i>=1; i--)
         bucket[k][i]=bucket[k][i-1];
      bucket[k][0]=0;
   }

   //store the leaf information
   leaves = new ulong[((ulong)1 << maxlevel)+1];
   for (k=0; k<(ulong)1<<maxlevel; k++)
      leaves[k] = bucket[maxlevel][k];
   leaves[k]=n;

   // compute wtree using the information about boundaries 
   for (i=0; i<n; i++) {
      value = Tools::GetField(D,maxlevel,i);
      for (k=0; k<maxlevel; k++)
            Tools::SetField(wtree,1,k*n+(bucket[k][value >> (maxlevel-k)])++,(value>>(maxlevel-k-1)) & (ulong)1);
   }
   
   
   // Create rank-structure for wtree
   bitrank = new BitRank(wtree,n*maxlevel,true);
   /*ulong logn = ((Tools::FloorLog2(n*maxlevel)+1)/2)*2;
   bitrank = new CompressedBitVector(wtree,n*maxlevel,true,W*W,W/2,W/2);
   delete wtree;*/
   
   // deallocate boundaries
   for (k=0; k<=maxlevel; k++)
      delete[] bucket[k];
   delete[] bucket;
}

WaveletTree::~WaveletTree() {
   //delete wtree;
   delete [] leaves;
   delete bitrank;
}
/* Version of rank not using array leaves[] 
ulong WaveletTree::rank(ulong c, ulong i) {
   ulong k;
   ulong j=i;
   ulong left = 0, right = n-1;
   ulong before;
   for (k=0;k<maxlevel;k++) {
      //std::cout << "[" << left << "," << j << "," << right << "]"; 
      if (n*k+left == 0) before=0;
      else before = bitrank->rank(n*k+left-1);
      if ((c >> (maxlevel-k-1)) & (ulong)1) { // go to the right
         j = bitrank->rank(n*k+left+j)-before-1;
         left += right-left+1-(bitrank->rank(n*k+right)-before);
      }
      else { // go to the left
         j -= bitrank->rank(n*k+left+j)-before;
	 right -= bitrank->rank(n*k+right)-before;
      }
      if (left > right) return 0;
   }
   // std::cout << "[" << left << "," << j << "," << right << "]";
   return j+1;	  
}
*/

/* Faster version of rank using leaves[] */
ulong WaveletTree::rank(ulong c, ulong i) {
   ulong k;
   ulong j=i;
   ulong left = 0, right = (ulong)1 << maxlevel;
   ulong before=0;
   
   // first round separately to avoid extra "if"  
   if ((c >> (maxlevel-1)) & (ulong)1) { // go to the right
      j = bitrank->rank(j)-1;
      left = right>>1; 
   }
   else { // go to the left
      j -= bitrank->rank(j);
      right = right>>1;
   }
   // then the next without the extra if
   for (k=1;k<maxlevel;k++) {
      before = bitrank->rank(n*k+leaves[left]-1);
      if ((c >> (maxlevel-k-1)) & (ulong)1) { // go to the right
         j = bitrank->rank(n*k+leaves[left]+j)-before-1;
         left = (right+left)>>1; 
      }
      else { // go to the left
         j -= bitrank->rank(n*k+leaves[left]+j)-before;
	 right = (right+left)>>1;
      }
      // can never happen: if (left > right) return 0;
   }
   return j+1;	  
}

ulong WaveletTree::select(ulong c, ulong j) {
   ulong left=c, right=c+1;
   ulong k, i=j-1, before;
   for (k=maxlevel-1; k>0;--k) {
       if ((c >> (maxlevel-k-1)) & (ulong)1) { // right side of parent
          left -= right-left;
	  before = bitrank->rank(n*k+leaves[left]-1);
	  i = bitrank->select(before+i+1)-n*k-leaves[left];
       }
       else { // left side of parent
          right +=right-left;
          before = n*k+leaves[left]-bitrank->rank(n*k+leaves[left]-1);
	  i = bitrank->select0(before+i+1)-n*k-leaves[left];
       }                         
   }
   // last level separately to avoid one extra if
   if ((c >> (maxlevel-1)) & (ulong)1) // right side of parent
      return bitrank->select(i+1);
   else // left side of parent
      return bitrank->select0(i+1);
}

bool WaveletTree::IsCharAtPos(ulong c, ulong i) {
   return charAtPos(i)==c;
}

ulong WaveletTree::charAtPos(ulong i) {
   ulong k;
   ulong j=i;
   ulong left = 0, right = n-1;
   ulong before;
   ulong c=0;
   for (k=0;k<maxlevel;k++) {
      //std::cout << "[" << left << "," << j << "," << right << "]"; 
      if (n*k+left == 0) before=0;
      else before = bitrank->rank(n*k+left-1);
      if (bitrank->IsBitSet(n*k+left+j)) { // go to the right
         c = c | ((ulong)1 << (maxlevel-k-1));
         j = bitrank->rank(n*k+left+j)-before-1;
         left += right-left+1-(bitrank->rank(n*k+right)-before);
      }
      else { // go to the left
         // c = c
         j -= bitrank->rank(n*k+left+j)-before;
	 right -= bitrank->rank(n*k+right)-before;
      }
      // should not happen: if (left > right) return 0;
   }
   return c;
}




